/**
 * Copyright (C) 2019  The Software Heritage developers
 * See the AUTHORS file at the top-level directory of this distribution
 * License: GNU Affero General Public License version 3, or any later version
 * See top-level LICENSE file for more information
 */

module.exports = [
  {
    'name': 'Apache License, Version 2.0',
    'url': 'http://www.apache.org/licenses/LICENSE-2.0',
    'magnet_link': 'magnet:?xt=urn:btih:8e4f440f4c65981c5bf93c76d35135ba5064d8b7&dn=apache-2.0.txt',
    'spdx_ids': ['Apache-2.0']
  },
  {
    'name': 'Artistic License 2.0',
    'url': 'http://www.perlfoundation.org/artistic_license_2_0',
    'magnet_link': 'magnet:?xt=urn:btih:54fd2283f9dbdf29466d2df1a98bf8f65cafe314&dn=artistic-2.0.txt',
    'spdx_ids': ['Artistic-2.0']
  },
  {
    'name': 'Boost Software License',
    'url': 'http://www.boost.org/LICENSE_1_0.txt',
    'magnet_link': 'magnet:?xt=urn:btih:89a97c535628232f2f3888c2b7b8ffd4c078cec0&dn=Boost-1.0.txt',
    'spdx_ids': ['BSL-1.0']
  },
  {
    'name': 'BSD 3-Clause License',
    'url': 'http://opensource.org/licenses/BSD-3-Clause',
    'magnet_link': 'magnet:?xt=urn:btih:c80d50af7d3db9be66a4d0a86db0286e4fd33292&dn=bsd-3-clause.txt',
    'spdx_ids': ['BSD-3-Clause']
  },
  {
    'name': 'CPAL 1.0',
    'url': 'http://opensource.org/licenses/cpal_1.0',
    'magnet_link': 'magnet:?xt=urn:btih:84143bc45939fc8fa42921d619a95462c2031c5c&dn=cpal-1.0.txt',
    'spdx_ids': ['CPAL-1.0']
  },
  {
    'name': 'Creative Commons CC0 1.0 Universal',
    'url': 'http://creativecommons.org/publicdomain/zero/1.0/legalcode',
    'magnet_link': 'magnet:?xt=urn:btih:90dc5c0be029de84e523b9b3922520e79e0e6f08&dn=cc0.txt',
    'spdx_ids': ['CC0-1.0']
  },
  {
    'name': 'Eclipse Public License 1.0',
    'url': 'http://www.eclipse.org/legal/epl-v10.html',
    'magnet_link': 'magnet:?xt=urn:btih:4c6a2ad0018cd461e9b0fc44e1b340d2c1828b22&dn=epl-1.0.txt',
    'spdx_ids': ['EPL-1.0']
  },
  {
    'name': 'MIT License',
    'url': 'http://www.jclark.com/xml/copying.txt',
    'magnet_link': 'magnet:?xt=urn:btih:d3d9a9a6595521f9666a5e94cc830dab83b65699&dn=expat.txt',
    'spdx_ids': ['MIT']
  },
  {
    'name': 'FreeBSD License',
    'url': 'http://www.freebsd.org/copyright/freebsd-license.html',
    'magnet_link': 'magnet:?xt=urn:btih:87f119ba0b429ba17a44b4bffcab33165ebdacc0&dn=freebsd.txt',
    'spdx_ids': ['BSD-2-Clause-FreeBSD']
  },
  {
    'name': 'GNU General Public License (GPL) version 2',
    'url': 'http://www.gnu.org/licenses/gpl-2.0.html',
    'magnet_link': 'magnet:?xt=urn:btih:cf05388f2679ee054f2beb29a391d25f4e673ac3&dn=gpl-2.0.txt',
    'spdx_ids': ['GPL-2.0-only', 'GPL-2.0-or-later', 'GPL-2.0+', 'GPL-2.0']
  },
  {
    'name': 'GNU General Public License (GPL) version 3',
    'url': 'http://www.gnu.org/licenses/gpl-3.0.html',
    'magnet_link': 'magnet:?xt=urn:btih:1f739d935676111cfff4b4693e3816e664797050&dn=gpl-3.0.txt',
    'spdx_ids': ['GPL-3.0-only', 'GPL-3.0-or-later', 'GPL-3.0+', 'GPL-3.0']
  },
  {
    'name': 'GNU Lesser General Public License, version 2.1',
    'url': 'http://www.gnu.org/licenses/lgpl-2.1.html',
    'magnet_link': 'magnet:?xt=urn:btih:5de60da917303dbfad4f93fb1b985ced5a89eac2&dn=lgpl-2.1.txt',
    'spdx_ids': ['LGPL-2.1-only', 'LGPL-2.1-or-later', 'LGPL-2.1+', 'LGPL-2.1']
  },
  {
    'name': 'GNU Lesser General Public License, version 3',
    'url': 'http://www.gnu.org/licenses/lgpl-3.0.html',
    'magnet_link': 'magnet:?xt=urn:btih:0ef1b8170b3b615170ff270def6427c317705f85&dn=lgpl-3.0.txt',
    'spdx_ids': ['LGPL-3.0-only', 'LGPL-3.0-or-later', 'LGPL-3.0+', 'LGPL-3.0']
  },
  {
    'name': 'GNU Affero General Public License, version 3',
    'url': 'http://www.gnu.org/licenses/agpl-3.0.html',
    'magnet_link': 'magnet:?xt=urn:btih:0b31508aeb0634b347b8270c7bee4d411b5d4109&dn=agpl-3.0.txt',
    'spdx_ids': ['AGPL-3.0-only', 'AGPL-3.0-or-later', 'AGPL-3.0+', 'AGPL-3.0']
  },
  {
    'name': 'The ISC License',
    'url': 'https://www.isc.org/downloads/software-support-policy/isc-license/',
    'magnet_link': 'magnet:?xt=urn:btih:b8999bbaf509c08d127678643c515b9ab0836bae&dn=ISC.txt',
    'spdx_ids': ['ISC']
  },
  {
    'name': 'Mozilla Public License 2.0',
    'url': 'http://www.mozilla.org/MPL/2.0',
    'magnet_link': 'magnet:?xt=urn:btih:3877d6d54b3accd4bc32f8a48bf32ebc0901502a&dn=mpl-2.0.txt',
    'spdx_ids': ['MPL-2.0']
  },
  {
    'name': 'Universal Permissive License',
    'url': 'https://oss.oracle.com/licenses/upl/',
    'magnet_link': 'magnet:?xt=urn:btih:478974f4d41c3fa84c4befba25f283527fad107d&dn=upl-1.0.txt',
    'spdx_ids': ['UPL-1.0']
  },
  {
    'name': 'WTFPL',
    'url': 'http://www.wtfpl.net/txt/copying/',
    'magnet_link': 'magnet:?xt=urn:btih:723febf9f6185544f57f0660a41489c7d6b4931b&dn=wtfpl.txt',
    'spdx_ids': ['WTFPL']
  },
  {
    'name': 'Unlicense',
    'url': 'http://unlicense.org/UNLICENSE',
    'magnet_link': 'magnet:?xt=urn:btih:5ac446d35272cc2e4e85e4325b146d0b7ca8f50c&dn=unlicense.txt',
    'spdx_ids': ['Unlicense']
  },
  {
    'name': 'X11 License',
    'url': 'http://www.xfree86.org/3.3.6/COPYRIGHT2.html#3',
    'magnet_link': 'magnet:?xt=urn:btih:5305d91886084f776adcf57509a648432709a7c7&dn=x11.txt',
    'spdx_ids': ['X11']
  },
  {
    'name': 'XFree86 License',
    'url': 'http://www.xfree86.org/current/LICENSE4.html',
    'magnet_link': 'magnet:?xt=urn:btih:12f2ec9e8de2a3b0002a33d518d6010cc8ab2ae9&dn=xfree86.txt',
    'spdx_ids': ['XFree86-1.1']
  }
];
