(()=>{"use strict";var e={d:(t,r)=>{for(var o in r)e.o(r,o)&&!e.o(t,o)&&Object.defineProperty(t,o,{enumerable:!0,get:r[o]})}};e.g=function(){if("object"==typeof globalThis)return globalThis;try{return this||new Function("return this")()}catch(e){if("object"==typeof window)return window}}(),e.o=(e,t)=>Object.prototype.hasOwnProperty.call(e,t),e.r=e=>{"undefined"!=typeof Symbol&&Symbol.toStringTag&&Object.defineProperty(e,Symbol.toStringTag,{value:"Module"}),Object.defineProperty(e,"__esModule",{value:!0})};var t={};e.r(t),e.d(t,{ExtpfeilConfiguration:()=>h});const r=("undefined"!=typeof window?window:e.g).MathJax._.components.global,o=(r.GLOBAL,r.isObject,r.combineConfig,r.combineDefaults,r.combineWithMathJax),a=(r.MathJax,MathJax._.input.tex.HandlerTypes),n=a.ConfigurationType,i=a.HandlerType,m=MathJax._.input.tex.Configuration,x=m.Configuration,w=(m.ConfigurationHandler,m.ParserConfiguration,MathJax._.input.tex.TokenMap),c=(w.parseResult,w.AbstractTokenMap,w.RegExpMap,w.AbstractParseMap,w.CharacterMap,w.DelimiterMap,w.MacroMap,w.CommandMap),u=(w.EnvironmentMap,MathJax._.input.tex.ams.AmsMethods.AmsMethods),p=MathJax._.input.tex.newcommand.NewcommandUtil,s=(p.NewcommandTables,p.NewcommandPriority,p.NewcommandUtil),d=MathJax._.input.tex.newcommand.NewcommandConfiguration,l=d.NewcommandConfig,f=(d.NewcommandConfiguration,MathJax._.input.tex.TexError.default),g={NewExtArrow(e,t){let r=e.GetArgument(t);const o=e.GetArgument(t),a=e.GetArgument(t);if(!r.match(/^\\([a-z]+|.)$/i))throw new f("NewextarrowArg1","First argument to %1 must be a control sequence name",t);if(!o.match(/^(\d+),(\d+)$/))throw new f("NewextarrowArg2","Second argument to %1 must be two integers separated by a comma",t);if(!a.match(/^(\d+|0x[0-9A-F]+)$/i))throw new f("NewextarrowArg3","Third argument to %1 must be a unicode character number",t);r=r.substring(1);const n=o.split(",");s.addMacro(e,r,g.xArrow,[parseInt(a),parseInt(n[0]),parseInt(n[1])]),e.Push(e.itemFactory.create("null"))},xArrow:u.xArrow};new c("extpfeil",{xtwoheadrightarrow:[g.xArrow,8608,12,16],xtwoheadleftarrow:[g.xArrow,8606,17,13],xmapsto:[g.xArrow,8614,6,7],xlongequal:[g.xArrow,61,7,7],xtofrom:[g.xArrow,8644,12,12],Newextarrow:g.NewExtArrow});const h=x.create("extpfeil",{[n.HANDLER]:{[i.MACRO]:["extpfeil"]},[n.CONFIG]:l});MathJax.loader&&MathJax.loader.checkVersion("[tex]/extpfeil","4.1.1","tex-extension"),o({_:{input:{tex:{extpfeil:{ExtpfeilConfiguration:t}}}}})})();