#include <iostream>
#include <unordered_map>
#include <vector>

using namespace std;

int main(int argc, char *argv[]) {

  /* An annoying "Hello World" example */
  for (auto i = 0; i < 0xFFFF; i++)
    cout << "Hello, World!" << endl;

  char c = '\n';
  unordered_map<string, vector<string>> m;
  m["key"] = "\\\\"; // this is an error

  return -2e3 + 12l;
}