(()=>{"use strict";var e={d:(t,a)=>{for(var n in a)e.o(a,n)&&!e.o(t,n)&&Object.defineProperty(t,n,{enumerable:!0,get:a[n]})}};e.g=function(){if("object"==typeof globalThis)return globalThis;try{return this||new Function("return this")()}catch(e){if("object"==typeof window)return window}}(),e.o=(e,t)=>Object.prototype.hasOwnProperty.call(e,t),e.r=e=>{"undefined"!=typeof Symbol&&Symbol.toStringTag&&Object.defineProperty(e,Symbol.toStringTag,{value:"Module"}),Object.defineProperty(e,"__esModule",{value:!0})};var t={};e.r(t),e.d(t,{default:()=>x});var a={};e.r(a),e.d(a,{AmsCdConfiguration:()=>T});const n=("undefined"!=typeof window?window:e.g).MathJax._.components.global,o=(n.GLOBAL,n.isObject,n.combineConfig,n.combineDefaults,n.combineWithMathJax),r=(n.MathJax,MathJax._.input.tex.HandlerTypes),i=r.ConfigurationType,s=r.HandlerType,c=MathJax._.input.tex.Configuration,m=c.Configuration,d=(c.ConfigurationHandler,c.ParserConfiguration,MathJax._.input.tex.TokenMap),l=(d.parseResult,d.AbstractTokenMap,d.RegExpMap,d.AbstractParseMap,d.CharacterMap,d.DelimiterMap,d.MacroMap),p=d.CommandMap,h=d.EnvironmentMap,u=MathJax._.input.tex.ParseMethods.default,f=MathJax._.input.tex.TexParser.default,w=MathJax._.input.tex.base.BaseConfiguration,g=w.Other,C=(w.BaseTags,w.BaseConfiguration,MathJax._.core.MmlTree.MmlNode),M=C.TEXCLASS,y=(C.TEXCLASSNAMES,C.MATHVARIANTS,C.indentAttributes,C.AbstractMmlNode,C.AbstractMmlTokenNode,C.AbstractMmlLayoutNode,C.AbstractMmlBaseNode,C.AbstractMmlEmptyNode,C.TextNode,C.XMLNode,MathJax._.input.tex.NodeUtil.default),b={CD(e,t){e.Push(t);const a=e.itemFactory.create("array"),n=e.configuration.options.amscd;return a.setProperties({minw:e.stack.env.CD_minw||n.harrowsize,minh:e.stack.env.CD_minh||n.varrowsize}),a.arraydef={columnalign:"center",columnspacing:n.colspace,rowspacing:n.rowspace,displaystyle:!0},a},arrow(e,t){const a=e.i,n=e.GetNext();if(!n.match(/[><VA.|=]/))return e.i=a,g(e,t);e.i++;let o=e.stack.Top();o.isKind("array")&&!o.Size()||(b.cell(e,t),o=e.stack.Top());const r=o,i=r.table.length%2==1;let s,c=(r.row.length+(i?0:1))%2;for(;c;)b.cell(e,t),c--;const m={minsize:r.getProperty("minw"),stretchy:!0},d={minsize:r.getProperty("minh"),stretchy:!0,symmetric:!0,lspace:0,rspace:0};if("|"===n)s=e.create("token","mo",d,"∥");else if("="===n)s=e.create("token","mo",m,"=");else if("."!==n){const a={">":"→","<":"←",V:"↓",A:"↑"}[n];let o=e.GetUpTo(t+n,n);const i=e.GetUpTo(t+n,n);if(">"===n||"<"===n){s=e.create("token","mo",m,a),o||(o="\\kern "+r.getProperty("minw"));const t={width:"+.67em",lspace:".33em"};s=e.create("node","munderover",[s]);const n=new f(o,e.stack.env,e.configuration).mml(),c=e.create("node","mpadded",[n],t);if(y.setAttribute(c,"voffset",".1em"),y.setChild(s,s.over,c),i){const a=new f(i,e.stack.env,e.configuration).mml();y.setChild(s,s.under,e.create("node","mpadded",[a],t))}e.configuration.options.amscd.hideHorizontalLabels&&(s=e.create("node","mpadded",[s],{depth:0,height:".67em"}))}else{const t=e.create("token","mo",d,a);s=t,(o||i)&&(s=e.create("node","mrow"),o&&y.appendChildren(s,[new f("\\scriptstyle\\raise.125em{\\vcenter{\\llap{"+o+"}}}",e.stack.env,e.configuration).mml()]),t.texClass=M.ORD,y.appendChildren(s,[t]),i&&y.appendChildren(s,[new f("\\scriptstyle\\raise.125em{\\vcenter{\\rlap{"+i+"}}}",e.stack.env,e.configuration).mml()]))}}s&&e.Push(s),b.cell(e,t)},cell(e,t){const a=e.stack.Top();(a.table||[]).length%2==0&&0===(a.row||[]).length&&e.Push(e.create("node","mpadded",[],{height:"8.5pt",depth:"2pt"})),e.Push(e.itemFactory.create("cell").setProperties({isEntry:!0,name:t}))},minCDarrowwidth(e,t){e.stack.env.CD_minw=e.GetDimen(t)},minCDarrowheight(e,t){e.stack.env.CD_minh=e.GetDimen(t)}},x=b;new h("amscd_environment",u.environment,{CD:x.CD}),new p("amscd_macros",{minCDarrowwidth:x.minCDarrowwidth,minCDarrowheight:x.minCDarrowheight}),new l("amscd_special",{"@":x.arrow});const T=m.create("amscd",{[i.HANDLER]:{[s.CHARACTER]:["amscd_special"],[s.MACRO]:["amscd_macros"],[s.ENVIRONMENT]:["amscd_environment"]},[i.OPTIONS]:{amscd:{colspace:"5pt",rowspace:"5pt",harrowsize:"2.75em",varrowsize:"1.75em",hideHorizontalLabels:!1}}});MathJax.loader&&MathJax.loader.checkVersion("[tex]/amscd","4.1.1","tex-extension"),o({_:{input:{tex:{amscd:{AmsCdConfiguration:a,AmsCdMethods:t}}}}})})();