(()=>{"use strict";var t={d:(e,a)=>{for(var o in a)t.o(a,o)&&!t.o(e,o)&&Object.defineProperty(e,o,{enumerable:!0,get:a[o]})}};t.g=function(){if("object"==typeof globalThis)return globalThis;try{return this||new Function("return this")()}catch(t){if("object"==typeof window)return window}}(),t.o=(t,e)=>Object.prototype.hasOwnProperty.call(t,e),t.r=t=>{"undefined"!=typeof Symbol&&Symbol.toStringTag&&Object.defineProperty(t,Symbol.toStringTag,{value:"Module"}),Object.defineProperty(t,"__esModule",{value:!0})};var e={};t.r(e),t.d(e,{ActionConfiguration:()=>h,ActionMethods:()=>d});const a=("undefined"!=typeof window?window:t.g).MathJax._.components.global,o=(a.GLOBAL,a.isObject,a.combineConfig,a.combineDefaults,a.combineWithMathJax),n=(a.MathJax,MathJax._.input.tex.HandlerTypes),i=n.ConfigurationType,r=n.HandlerType,c=MathJax._.input.tex.Configuration,p=c.Configuration,s=(c.ConfigurationHandler,c.ParserConfiguration,MathJax._.input.tex.TexParser.default),l=MathJax._.input.tex.TokenMap,u=(l.parseResult,l.AbstractTokenMap,l.RegExpMap,l.AbstractParseMap,l.CharacterMap,l.DelimiterMap,l.MacroMap,l.CommandMap),g=(l.EnvironmentMap,MathJax._.input.tex.base.BaseMethods),M=(g.splitAlignArray,g.default),d={Toggle(t,e){const a=[];let o;for(;"\\endtoggle"!==(o=t.GetArgument(e));)a.push(new s(o,t.stack.env,t.configuration).mml());t.Push(t.create("node","maction",a,{actiontype:"toggle"}))},Mathtip(t,e){const a=t.ParseArg(e),o=t.ParseArg(e);t.Push(t.create("node","maction",[a,o],{actiontype:"tooltip"}))},Macro:M.Macro};new u("action-macros",{toggle:d.Toggle,mathtip:d.Mathtip,texttip:[d.Macro,"\\mathtip{#1}{\\text{#2}}",2]});const h=p.create("action",{[i.HANDLER]:{[r.MACRO]:["action-macros"]}});MathJax.loader&&MathJax.loader.checkVersion("[tex]/action","4.1.1","tex-extension"),o({_:{input:{tex:{action:{ActionConfiguration:e}}}}})})();