# Copyright (C) 2019-2024  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU Affero General Public License version 3, or any later version
# See top-level LICENSE file for more information

from pytest_django.asserts import assertContains, assertNotContains, assertTemplateUsed

assert_template_used = assertTemplateUsed
assert_contains = assertContains
assert_not_contains = assertNotContains
