<%#
  Copyright (C) 2023  The Software Heritage developers
  See the AUTHORS file at the top-level directory of this distribution
  License: GNU Affero General Public License version 3, or any later version
  See top-level LICENSE file for more information
%>

Dear forge administrator,

We are happy to announce to you that your forge is now successfully
archived.

The forge repositories are now available in the archive :
https://archive.softwareheritage.org/browse/search/?q=<%= forgeUrl %>&with_visit=true&with_content=true

Kind regards,

The Software Heritage team