(()=>{"use strict";var e={d:(o,t)=>{for(var r in t)e.o(t,r)&&!e.o(o,r)&&Object.defineProperty(o,r,{enumerable:!0,get:t[r]})}};e.g=function(){if("object"==typeof globalThis)return globalThis;try{return this||new Function("return this")()}catch(e){if("object"==typeof window)return window}}(),e.o=(e,o)=>Object.prototype.hasOwnProperty.call(e,o),e.r=e=>{"undefined"!=typeof Symbol&&Symbol.toStringTag&&Object.defineProperty(e,Symbol.toStringTag,{value:"Module"}),Object.defineProperty(e,"__esModule",{value:!0})};var o={};e.r(o),e.d(o,{NoErrorsConfiguration:()=>u});const t=("undefined"!=typeof window?window:e.g).MathJax._.components.global,r=(t.GLOBAL,t.isObject,t.combineConfig,t.combineDefaults,t.combineWithMathJax),n=(t.MathJax,MathJax._.input.tex.HandlerTypes),a=n.ConfigurationType,i=(n.HandlerType,MathJax._.input.tex.Configuration),c=i.Configuration;i.ConfigurationHandler,i.ParserConfiguration;const u=c.create("noerrors",{[a.NODES]:{error:function(e,o,t,r){const n=e.create("token","mtext",{},r.replace(/\n/g," "));return e.create("node","merror",[n],{"data-mjx-error":o,title:o})}}});MathJax.loader&&MathJax.loader.checkVersion("[tex]/noerrors","4.1.1","tex-extension"),r({_:{input:{tex:{noerrors:{NoErrorsConfiguration:o}}}}})})();