{% extends "browse.html" %}

{% comment %}
Copyright (C) 2017-2024  The Software Heritage developers
See the AUTHORS file at the top-level directory of this distribution
License: GNU Affero General Public License version 3, or any later version
See top-level LICENSE file for more information
{% endcomment %}

{% block navbar-content %}
  <h4>Download archived software</h4>
{% endblock navbar-content %}

{% block browse-content %}
  <p>
    This interface enables you to track the status of the different Software Heritage
    Vault cooking tasks created while browsing the archive.
  </p>
  <p>
    Once a cooking task is finished, a link will be made available in order to
    download the associated archive.
  </p>
  <button type="button"
          class="btn btn-secondary btn-sm"
          id="vault-remove-tasks">Remove selected tasks</button>
  <div class="table-responsive mt-3">
    <table class="table swh-table swh-table-striped swh-vault-table"
           id="vault-cooking-tasks">
      <thead>
        <tr>
          <th>
            <div class="form-check">
              <input type="checkbox"
                     class="form-check-input"
                     id="vault-tasks-toggle-selection" />
              <label class="form-check-label" for="vault-tasks-toggle-selection"></label>
            </div>
          </th>
          <th style="width: 300px">Origin</th>
          <th style="width: 100px">Bundle type</th>
          <th>Object info</th>
          <th style="width: 250px">Cooking status</th>
          <th style="width: 120px"></th>
        </tr>
      </thead>
      <tbody>
      </tbody>
    </table>
  </div>
  {% include "./includes/vault-common.html" %}
  <script>
    swh.webapp.initPage('vault');
    swh.vault.initUi();
  </script>
{% endblock browse-content %}
