(()=>{"use strict";var e={d:(a,t)=>{for(var p in t)e.o(t,p)&&!e.o(a,p)&&Object.defineProperty(a,p,{enumerable:!0,get:t[p]})}};e.g=function(){if("object"==typeof globalThis)return globalThis;try{return this||new Function("return this")()}catch(e){if("object"==typeof window)return window}}(),e.o=(e,a)=>Object.prototype.hasOwnProperty.call(e,a),e.r=e=>{"undefined"!=typeof Symbol&&Symbol.toStringTag&&Object.defineProperty(e,Symbol.toStringTag,{value:"Module"}),Object.defineProperty(e,"__esModule",{value:!0})};var a={};e.r(a),e.d(a,{UpgreekConfiguration:()=>l});const t=("undefined"!=typeof window?window:e.g).MathJax._.components.global,p=(t.GLOBAL,t.isObject,t.combineConfig,t.combineDefaults,t.combineWithMathJax),n=(t.MathJax,MathJax._.input.tex.HandlerTypes),o=n.ConfigurationType,i=n.HandlerType,r=MathJax._.input.tex.Configuration,u=r.Configuration,s=(r.ConfigurationHandler,r.ParserConfiguration,MathJax._.input.tex.TokenMap),c=(s.parseResult,s.AbstractTokenMap,s.RegExpMap,s.AbstractParseMap,s.CharacterMap),g=(s.DelimiterMap,s.MacroMap,s.CommandMap,s.EnvironmentMap,MathJax._.input.tex.TexConstants.TexConstant);new c("upgreek",function(e,a){const t=a.attributes||{};t.mathvariant=g.Variant.NORMAL;const p=e.create("token","mi",t,a.char);e.Push(p)},{upalpha:"α",upbeta:"β",upgamma:"γ",updelta:"δ",upepsilon:"ϵ",upzeta:"ζ",upeta:"η",uptheta:"θ",upiota:"ι",upkappa:"κ",uplambda:"λ",upmu:"μ",upnu:"ν",upxi:"ξ",upomicron:"ο",uppi:"π",uprho:"ρ",upsigma:"σ",uptau:"τ",upupsilon:"υ",upphi:"ϕ",upchi:"χ",uppsi:"ψ",upomega:"ω",upvarepsilon:"ε",upvartheta:"ϑ",upvarpi:"ϖ",upvarrho:"ϱ",upvarsigma:"ς",upvarphi:"φ",Upgamma:"Γ",Updelta:"Δ",Uptheta:"Θ",Uplambda:"Λ",Upxi:"Ξ",Uppi:"Π",Upsigma:"Σ",Upupsilon:"Υ",Upphi:"Φ",Uppsi:"Ψ",Upomega:"Ω"});const l=u.create("upgreek",{[o.HANDLER]:{[i.MACRO]:["upgreek"]}});MathJax.loader&&MathJax.loader.checkVersion("[tex]/upgreek","4.1.1","tex-extension"),p({_:{input:{tex:{upgreek:{UpgreekConfiguration:a}}}}})})();