# Generated by Django 2.2.24 on 2022-03-21 16:37

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("swh_web_add_forge_now", "0002_authorized_null_comment"),
    ]

    operations = [
        migrations.AddField(
            model_name="request",
            name="submitter_forward_username",
            field=models.BooleanField(default=False),
        ),
    ]
