/**
 * Copyright (C) 2018-2021  The Software Heritage developers
 * See the AUTHORS file at the top-level directory of this distribution
 * License: GNU Affero General Public License version 3, or any later version
 * See top-level LICENSE file for more information
 */

.swh-diff-lines-info {
    background-color: rgb(0 0 255 / 10%) !important;
}

.swh-diff-added-line {
    background-color: rgb(0 255 0 / 10%) !important;
}

.swh-diff-removed-line {
    background-color: rgb(255 0 0 / 10%) !important;
}

span.no-nl-marker {
    position: relative;
    color: #cb2431;
    vertical-align: middle;
}

span.no-nl-marker svg {
    vertical-align: text-bottom;
}

span.no-nl-marker svg path {
    fill: currentcolor;
}

.swh-revision-log-entry-id {
    min-width: 110px;
    max-width: 110px;
    width: 110px;
}

.swh-revision-log-entry-author {
    min-width: 160px;
    max-width: 160px;
    width: 160px;
}

.swh-revision-log-entry-date {
    min-width: 200px;
    max-width: 200px;
    width: 200px;
}

.swh-revision-log-entry-commit-date {
    min-width: 200px;
    max-width: 200px;
    width: 200px;
}
