{% comment %}
Copyright (C) 2021-2023  The Software Heritage developers
See the AUTHORS file at the top-level directory of this distribution
License: GNU Affero General Public License version 3, or any later version
See top-level LICENSE file for more information
{% endcomment %}

{% load static %}

<link rel="icon"
      href="{% static 'img/icons/swh-logo-32x32.png' %}"
      sizes="32x32" />
<link rel="icon"
      href="{% static 'img/icons/swh-logo-archive-192x192.png' %}"
      sizes="192x192" />
<link rel="apple-touch-icon-precomposed"
      href="{% static 'img/icons/swh-logo-archive-180x180.png' %}" />
<link rel="search"
      type="application/opensearchdescription+xml"
      title="Software Heritage archive of public source code"
      href="{% static 'xml/swh-opensearch.xml' %}" />
<meta name="msapplication-TileImage"
      content="{% static 'img/icons/swh-logo-archive-270x270.png' %}" />
