<%#
  Copyright (C) 2022  The Software Heritage developers
  See the AUTHORS file at the top-level directory of this distribution
  License: GNU Affero General Public License version 3, or any later version
  See top-level LICENSE file for more information
%>
Hello <%= forgeAdmin %>,

The mission of Software Heritage is to collect, preserve and share all
the publicly available source code: https://www.softwareheritage.org

We would like to add the code hosted at the URL below to the list of
software origins that are regularly archived, and we understand that
you are or know the contact person for this site.

<%= forgeUrl %>

In order to archive the code contents, we will have to periodically
pull the public repositories it contains and clone them into the
Software Heritage archive. FAQs for our processes are available:

https://docs.softwareheritage.org/user/faq/#add-forge-now
https://www.softwareheritage.org/faq/

Please let us know if there are any issues to consider before
we launch the archival of the public repositories hosted on your
infrastructure. Please use "Reply all" to ensure our system will
process your answer properly.

In the absence of an answer to this message, we will start to
archive your code in the coming weeks. Only the publicly
accessible repositories will be archived.

Thank you in advance for your help.

Kind regards,
The Software Heritage team
