#swh-token-error-message{color:red;font-weight:700}#swh-token-success-message{color:green;font-weight:700}#swh-bearer-token{overflow-wrap:break-word;white-space:break-spaces}
/*# sourceMappingURL=auth.21410a136651bcb378ae.css.map*/