{% comment %}
Copyright (C) 2019-2024  The Software Heritage developers
See the AUTHORS file at the top-level directory of this distribution
License: GNU Affero General Public License version 3, or any later version
See top-level LICENSE file for more information
{% endcomment %}

<iframe id="vault-download-iframe" style="display:none;"></iframe>
<div class="modal fade d-none"
     id="vault-recook-object-modal"
     tabindex="-1"
     role="dialog"
     aria-labelledby="vault-recook-object-modal-label"
     aria-hidden="true">
  <div class="modal-dialog">
    <div class="modal-content">
      <div class="modal-header">
        <h6 class="modal-title" id="vault-recook-object-modal-label">Download link has expired</h6>
        <button type="button"
                class="btn-close"
                data-bs-dismiss="modal"
                aria-label="Close"></button>
      </div>
      <div class="modal-body">
        <p>The requested archive is no longer available for download from the Software Heritage Vault.</p>
        <p>Do you want to cook it again ?</p>
      </div>
      <div class="modal-footer">
        <button type="button"
                class="btn btn-secondary btn-sm"
                data-bs-dismiss="modal">Cancel</button>
        <button type="button"
                class="btn btn-secondary btn-sm"
                onclick="swh.vault.recookObject()">Ok</button>
      </div>
    </div>
  </div>
</div>
