{% extends "./add-forge-common.html" %}

{% comment %}
Copyright (C) 2022-2023  The Software Heritage developers
See the AUTHORS file at the top-level directory of this distribution
License: GNU Affero General Public License version 3, or any later version
See top-level LICENSE file for more information
{% endcomment %}

{% block tab_content %}
  <div id="swh-add-forge-requests-help" class="tab-pane active mt-3">
    <p>To submit an "Add forge now" request, you have to provide the following details:</p>
    <ul>
      <li>
        <strong>Forge type:</strong> Type of the forge you would like to add.
        Supported forge types in software heritage currently are:
        <ul>
          <li>
            <code>cgit</code>, for <a href="https://git.zx2c4.com/cgit/">cgit</a> forges
          </li>
          <li>
            <code>forgejo</code>, for <a href="https://forgejo.org/">forgejo</a> forges
          </li>
          <li>
            <code>gitea</code>, for <a href="https://about.gitea.com/">gitea</a> forges
          </li>
          <li>
            <code>gitiles</code>, for <a href="https://github.com/google/gitiles">gitiles</a> forges
          </li>
          <li>
            <code>gitlab</code>, for <a href="https://about.gitlab.com/install/">gitlab</a> forges
          </li>
          <li>
            <code>gitweb</code>, for <a href="https://git-scm.com/book/en/v2/Git-on-the-Server-GitWeb">gitweb</a> forges
          </li>
          <li>
            <code>gogs</code>, for <a href="https://gogs.io/">gogs</a> forges
          </li>
          <li>
            <code>heptapod</code>, for <a href="https://heptapod.net/">heptapod</a> forges
          </li>
          <li>
            <code>stagit</code>, for <a href="https://codemadness.org/stagit.html">stagit</a> forges
          </li>
          ...
        </ul>
      </li>
      <li>
        <strong>Forge url:</strong> The URL of the selected forge. This must be unique. This must not be GitHub. This must be a canonical URL. This must be a base URL.
      </li>
      <li>
        <strong>Forge contact name:</strong> Contact name of the forge administrator
      </li>
      <li>
        <strong>Forge contact email:</strong> Contact email of the forge administrator. An email
        will be sent to the given address to notify about the request.
      </li>
      <li>
        <strong>Consent checkbox:</strong> This checkbox's purpose is to know whether we can
        explicitly mention the user's login within the email sent to the forge. If
        not checked, the user's name won't be mentioned in the email at all.
      </li>
      <li>
        <strong>Comment:</strong> (Optionally) For the user to mention something more about
        their request to the add-forge-now moderator.
      </li>
    </ul>
    <p class="mt-1">
      Once submitted, your "add forge" request can be in one
      of the following states
    </p>
    <ul>
      <li>
        <strong>Pending:</strong>
        The request was submitted and is waiting for a moderator
        to validate
      </li>
      <li>
        <strong>Waiting for feedback:</strong>
        The request was processed by a moderator and the forge was contacted.
      </li>
      <li>
        <strong>Feedback to handle:</strong>
        The forge has responded to the request and
        there is feedback to handle for the request.
      </li>
      <li>
        <strong>Accepted:</strong>
        The request has been accepted.
      </li>
      <li>
        <strong>Scheduled:</strong>
        The requested forge listing has been scheduled.
      </li>
      <li>
        <strong>First listing done:</strong>
        The first listing of the requested forge has been completed
      </li>
      <li>
        <strong>First origin loaded:</strong>
        The first repositories (or origins) from the requested forge have been loaded and archived.
      </li>
      <li>
        <strong>Rejected:</strong> The request is invalid. It is rejected by a moderator with an explanation.
      </li>
      <li>
        <strong>Unsuccessful:</strong>An issue has been encountered which currently prevents the full archival.
      </li>
      <li>
        <strong>Suspended:</strong>The forge listing is not supported yet.
      </li>
    </ul>
  </div>
{% endblock tab_content %}
