/**
 * Copyright (C) 2022  The Software Heritage developers
 * See the AUTHORS file at the top-level directory of this distribution
 * License: GNU Affero General Public License version 3, or any later version
 * See top-level LICENSE file for more information
 */

.request-actions {
    padding-right: 10px;
}

.details-text {
    padding-top: 5px;
}

.history-item {
    padding-bottom: 5px;
}
