(()=>{"use strict";var e={d:(t,n)=>{for(var a in n)e.o(n,a)&&!e.o(t,a)&&Object.defineProperty(t,a,{enumerable:!0,get:n[a]})}};e.g=function(){if("object"==typeof globalThis)return globalThis;try{return this||new Function("return this")()}catch(e){if("object"==typeof window)return window}}(),e.o=(e,t)=>Object.prototype.hasOwnProperty.call(e,t),e.r=e=>{"undefined"!=typeof Symbol&&Symbol.toStringTag&&Object.defineProperty(e,Symbol.toStringTag,{value:"Module"}),Object.defineProperty(e,"__esModule",{value:!0})};var t={};e.r(t),e.d(t,{CancelConfiguration:()=>x,CancelMethods:()=>g});const n=("undefined"!=typeof window?window:e.g).MathJax._.components.global,a=(n.GLOBAL,n.isObject,n.combineConfig,n.combineDefaults,n.combineWithMathJax),o=(n.MathJax,MathJax._.input.tex.HandlerTypes),i=o.ConfigurationType,r=o.HandlerType,c=MathJax._.input.tex.Configuration,l=c.Configuration,s=(c.ConfigurationHandler,c.ParserConfiguration,MathJax._.input.tex.TexConstants.TexConstant),p=MathJax._.input.tex.TokenMap,u=(p.parseResult,p.AbstractTokenMap,p.RegExpMap,p.AbstractParseMap,p.CharacterMap,p.DelimiterMap,p.MacroMap,p.CommandMap),d=(p.EnvironmentMap,MathJax._.input.tex.ParseUtil),f=(d.KeyValueDef,d.KeyValueTypes,d.ParseUtil),h=MathJax._.input.tex.enclose.EncloseConfiguration,M=h.ENCLOSE_OPTIONS,g=(h.EncloseMethods,h.EncloseConfiguration,{Cancel(e,t,n){const a=e.GetBrackets(t,""),o=e.ParseArg(t),i=f.keyvalOptions(a,M);i.notation=n,e.Push(e.create("node","menclose",[o],i))},CancelTo(e,t){const n=e.GetBrackets(t,"");let a=e.ParseArg(t);const o=e.ParseArg(t),i=f.keyvalOptions(n,M);i.notation=[s.Notation.UPDIAGONALSTRIKE,s.Notation.UPDIAGONALARROW,s.Notation.NORTHEASTARROW].join(" "),a=e.create("node","mpadded",[a],{depth:"-.1em",height:"+.1em",voffset:".1em"}),e.Push(e.create("node","msup",[e.create("node","menclose",[o],i),a]))}});new u("cancel",{cancel:[g.Cancel,s.Notation.UPDIAGONALSTRIKE],bcancel:[g.Cancel,s.Notation.DOWNDIAGONALSTRIKE],xcancel:[g.Cancel,s.Notation.UPDIAGONALSTRIKE+" "+s.Notation.DOWNDIAGONALSTRIKE],cancelto:g.CancelTo});const x=l.create("cancel",{[i.HANDLER]:{[r.MACRO]:["cancel"]}});MathJax.loader&&MathJax.loader.checkVersion("[tex]/cancel","4.1.1","tex-extension"),a({_:{input:{tex:{cancel:{CancelConfiguration:t}}}}})})();