{% extends "layout.html" %}

{% comment %}
Copyright (C) 2018-2024  The Software Heritage developers
See the AUTHORS file at the top-level directory of this distribution
License: GNU Affero General Public License version 3, or any later version
See top-level LICENSE file for more information
{% endcomment %}

{% load static %}

{% block page_title %}
  Logged out
{% endblock page_title %}

{% block navbar-content %}
  <h4>Logged out</h4>
{% endblock navbar-content %}

{% block content %}
  <p>Your authenticated session expired or you explicitly logged out.</p>
  <p>
    {% if 'next' in request.GET %}
      <a href="{% url login_url %}?next={{ request.GET.next|urlencode }}">Log in</a>
    {% else %}
      <a href="{% url login_url %}">Log in</a>
    {% endif %}
    again ?
  </p>
  {% if 'next' in request.GET %}
    <p>
      Or go back to
      <a href="{{ request.GET.next }}">previous page</a>.
    </p>
  {% endif %}
{% endblock content %}
